#define F_CPU 14745600UL 																		//    

#include <avr/io.h> 																			//  -
#include <avr/eeprom.h>																			//    EEPROM
#include <avr/wdt.h>																			//  
#include <stdio.h>	
#include <avr/interrupt.h>																		//  

#include "main.h"
#include "MC8U8O.h"																				// 
#include "LED.h"																				//   LED 
#include "Key.h"																				// 
#include "ADS1248.h"	

/*******************************************************************************************************************************************
												  0
*******************************************************************************************************************************************/
void Init_Tim0(void)
{
	TCCR0 = (1<<WGM01)|(1<<CS02)|(1<<CS00);								//  (1024)    (     )
	TCNT0 = 0;															//  
	OCR0 = 144;															//   (10 )
	TIMSK |= (1<<OCIE0);												//      
}

/*******************************************************************************************************************************************
													  0
*******************************************************************************************************************************************/
void Stop_Tim0(void)
{
	TCCR0 = 0;
	TIMSK &= ~(1<<OCIE0);
}


char String[17];																							//   
uint16_t cnt_tim0 = 0;																						//   1     0
uint8_t Calibrationmode=0;

extern uint32_t OFC1, OFC0, FSC1, FSC0;

#define  EEPROM_ADR_OFC0 (uint32_t*)0x00000200
#define  EEPROM_ADR_OFC1 (uint32_t*)0x00000204
#define  EEPROM_ADR_FSC0 (uint32_t*)0x00000208
#define  EEPROM_ADR_FSC1 (uint32_t*)0x0000020C

uint32_t dataformADC[3] = {0,0,0};
float temp, temp2;	
//========   =======//
ISR(TIMER0_COMP_vect)
{
	if (Calibrationmode == 0)																				//   
	switch(cnt_tim0)																						//   
	{
		case 100:	write_ADS1248(10);	break;																//      0 TI Type J
		case 200:																	
			dataformADC[0] = read_ADS1248();																//   
			sprintf(String, "CH0  %4X%4X", (uint16_t)(dataformADC[0]>>16),(uint16_t)(dataformADC[0]>>0));	//     
			LED_SetPos(0,0);																				//    0   0
			LED_Write_String(String);																		//     
			temp = (float)dataformADC[0]*70/0x7FFFFF;														//    (70     )
			sprintf(String, "CH0  %07d", (int16_t)(temp*100));												//        0.01
			LED_SetPos(1,0);																				//    1   0
			LED_Write_String(String);																		//      
			cnt_tim0 = 0;																					//  
			LED_RED_TUGGLE;
			break;
		
	}
	cnt_tim0++;																								//   
	
	TCNT0 = 0;																								//    
}

int main(void)																								// 
{

	DDR_Led_Red |= (1<<P_Led_Red);																		//    
		
	Init_ADS1248();																						// 
  	_delay_ms(200);
	Reset_ADS1248();																					//    
	_delay_ms(200);
	Init_Tim0();																						// 
	LED_Init();																							// 
	Key_Init(0);																						// 

	sprintf(String, "      ");																//    
	LED_SetPos(0,0);																					//    
	LED_Write_String(String);																			//    
	sprintf(String, "                ");																//    
	LED_SetPos(1,0);																					//    
	LED_Write_String(String);																			//    

	sei();																								// 
	
	while (1) 																							// 
	{
		if(Key(0)==1 && Key(1)== 1 && Key(2)== 1 && Key(3)== 1 && Calibrationmode == 0)					//   4      
		{
			_delay_ms(500);																				//   
			if (Key(0)==1 && Key(1)== 1 && Key(2)== 1 && Key(3)== 1)									//   4      
			{
				Calibrationmode=1;																		//      
				cli();																					// 
				_delay_ms(100);
				sprintf(String, "      ");													//        
				LED_SetPos(0,0);																		//        
				LED_Write_String(String);																//        
				sprintf(String, "          ");													//        
				LED_SetPos(1,0);																		//        
				LED_Write_String(String);																//        
				_delay_ms(500);
			}
		}
		
		if(Calibrationmode == 1)																		//     
		{																								//   
			uint8_t temp[10];																			//  
			temp[0] = Key(3)<<3|Key(2)<<2|Key(1)<<1|Key(0);												//    
			_delay_ms(500);																				//    
			temp[1] = Key(3)<<3|Key(2)<<2|Key(1)<<1|Key(0);												//    
			if (temp[0] == temp[1])																		//      
			{
				switch (temp[1])																		//    
				{
					case 0x0f:																			//   
						Calibrationmode=0;																//    
						_delay_ms(100);
						sprintf(String, "      ");											//         
						LED_SetPos(0,0);																//         
						LED_Write_String(String);														//         
						sprintf(String, "       ");											//         
						LED_SetPos(1,0);																//         
						LED_Write_String(String);														//         
						_delay_ms(500);
						sei();
						break;
					case 0b0001:																		//   0 
						_delay_ms(100);
						OFC0 = Calibrov_Min_TI(0);														//    0   0 TI type J
						_delay_ms(100);
						eeprom_write_dword(EEPROM_ADR_OFC0, OFC0);										//     eeprom
						_delay_ms(100);
						sprintf(String, " CH0  ");											//     	
						LED_SetPos(0,0);																//     
						LED_Write_String(String);														//     
						sprintf(String, " 0 %4X%4X",(uint16_t)(OFC0>>16),(uint16_t)(OFC0>>0));		//     
						LED_SetPos(1,0);																//     
						LED_Write_String(String);														//     
						break;
					case 0b0010:
						_delay_ms(100);
						FSC0 = Calibrov_Max_TI(0);														//      0 TI type J
						_delay_ms(100);
						eeprom_write_dword(EEPROM_ADR_FSC0, FSC0);										//     eeprom
						_delay_ms(100);
						sprintf(String, " CH0  ");											//     
						LED_SetPos(0,0);																//     
						LED_Write_String(String);														//     
						sprintf(String, " 0 %4X%4X",(uint16_t)(FSC0>>16),(uint16_t)(FSC0>>0));		//     
						LED_SetPos(1,0);																//     
						LED_Write_String(String);														//     
						break;
					case 0b0100:
						_delay_ms(100);
						OFC1 = Calibrov_Min_TI(1);														//    0   1 TI type J
						_delay_ms(100);
						eeprom_write_dword(EEPROM_ADR_OFC1, OFC1);										//     eeprom
						_delay_ms(100);
						sprintf(String, " CH1  ");											//     
						LED_SetPos(0,0);																//     
						LED_Write_String(String);														//     
						sprintf(String, " 1 %4X%4X",(uint16_t)(OFC1>>16),(uint16_t)(OFC1>>0));		//     
						LED_SetPos(1,0);																//     
						LED_Write_String(String);														//     
						break;
					case 0b1000:
						_delay_ms(100);
						FSC1 = Calibrov_Max_TI(1);														//      1 TI type J
						_delay_ms(100);
						eeprom_write_dword(EEPROM_ADR_FSC1, FSC1);										//     eeprom
						_delay_ms(100);																	//     
						sprintf(String, " CH1  ");											//     
						LED_SetPos(0,0);																//     
						LED_Write_String(String);														//     
						sprintf(String, " 1 %4X%4X",(uint16_t)(FSC1>>16),(uint16_t)(FSC1>>0));		//     
						LED_SetPos(1,0);																//     
						LED_Write_String(String);														//     
						break;																
				}
			}
		}
		
	}
}

